// -*- C++ -*-
/*!
 * @file  WiiBalanceBoardTranslator.cpp
 * @brief WiiBalanceBoardTranslator
 * @date $Date$
 *
 * @author 〒135-8548
 * 東京都江東区豊洲3-7-5　芝浦工業大学　研究棟　11-Q-32
 * 水川(HRI)研究室
 * 代表者:水川　真   (Emai:mizukawa<at>sic.shibaura-it.ac.jp)
 * 作成者;田中　基雅(Email:m108081<at>shibaura-it.ac.jp)
 *          藤田　恒彦(Email:m109077<at>shibaura-it.ac.jp)
 *          鷹栖　尭大（Email:m109056<at>shibaura-it.ac.jp）
 *
 * WiiBalanceBoardTranslatorについての著作権は，芝浦工業大学 水川研究室に帰属しま
 * す．
 * 非営利目的利用は自由です．
 *
 * $Id$
 */

#include "WiiBalanceBoardTranslator.h"

// Module specification
// <rtc-template block="module_spec">
static const char* wiibalanceboardtranslator_spec[] =
  {
    "implementation_id", "WiiBalanceBoardTranslator",
    "type_name",         "WiiBalanceBoardTranslator",
    "description",       "BalanceBoard Translator Component",
    "version",           "1.0.0",
    "vendor",            "Shibaura Institute of Technology",
    "category",          "RemoteController",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    //"exec_cxt.periodic.rate", "1.0",
    // Configuration variables
    "conf.default.debug", "0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
WiiBalanceBoardTranslator::WiiBalanceBoardTranslator(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_WiiDataIn("WiiData", m_WiiData),
    m_WiiConvertOut("WiiConvert", m_WiiConvert),
    m_WiiInfoPort("WiiInfo")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
WiiBalanceBoardTranslator::~WiiBalanceBoardTranslator()
{
}


/*!
 * Configurationの初期化を行う
 */
RTC::ReturnCode_t WiiBalanceBoardTranslator::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("WiiData", m_WiiDataIn);
  
  // Set OutPort buffer
  registerOutPort("WiiConvert", m_WiiConvertOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_WiiInfoPort.registerConsumer("WiiInfo", "WiiInfo", m_WiiInfo);
  
  // Set CORBA Service Ports
  registerPort(m_WiiInfoPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("debug", m_debug, "0");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiBalanceBoardTranslator::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiBalanceBoardTranslator::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiBalanceBoardTranslator::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * キャリブレーションデータを取得し，WiiRemoteAgentに送信する．
 */

RTC::ReturnCode_t WiiBalanceBoardTranslator::onActivated(RTC::UniqueId ec_id)
{
	Calibration_var CalibList;
	BYTE status = 0x00;
	BYTE buff[255];

	status = 0x00;
	while(1)
	{
		CalibList = m_WiiInfo->get_balance_calib();
		status = CalibList[0];
		if(status == 0x21)
		{
			break;
		}
		Sleep(50);
	}

	for(int i = 0; i < 44 ; i++){
		buff[i] = CalibList[i];
	}
	wiibb.SetWiiExtensionCalibration(buff);
	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiBalanceBoardTranslator::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * WiiRemoteAgentから生データが入力された時に，それを整形データに変換し，出力する
 * ．
 * 整形データはTimedDoubleSeq型で出力する．
 * データの内訳を以下に記す．
 * [0] 電源ボタンの状態(BOOL 値)
 * [1] X軸の重心座標
 * [2] Y軸の重心座標
 * [3] 左上にかかる重量データ
 * [4] 右上にかかる重量データ
 * [5] 右下にかかる重量データ
 * [5] 左下にかかる重量データ
 * [7] 左上センサの生データ
 * [8] 右上センサの生データ
 * [9] 右下センサの生データ
 * [10] 左下センサの生データ
 */

RTC::ReturnCode_t WiiBalanceBoardTranslator::onExecute(RTC::UniqueId ec_id)
{
	BYTE data[22];
	if(m_WiiDataIn.isNew())
	{
		m_WiiDataIn.read();
		for(int i = 0 ; i < 22 ; i++)
		{
			data[i] = m_WiiData.data[i];	
		}

		wiibb.ParseButtons(data);
		wiibb.ParseExtension(data);

		m_WiiConvert.data.length(11);
		m_WiiConvert.data[0] = (double)wiibb.mWiiRemoteState.ButtonState.A;
		m_WiiConvert.data[1] = (double)wiibb.BalanceBoardState.CenterOfGravity.X;
		m_WiiConvert.data[2] = (double)wiibb.BalanceBoardState.CenterOfGravity.Y;
		m_WiiConvert.data[3] = wiibb.BalanceBoardState.SensorValuesKg.TopLeft;
		m_WiiConvert.data[4] = wiibb.BalanceBoardState.SensorValuesKg.TopRight;
		m_WiiConvert.data[5] = wiibb.BalanceBoardState.SensorValuesKg.BottomRight;
		m_WiiConvert.data[6] = wiibb.BalanceBoardState.SensorValuesKg.BottomLeft;
		m_WiiConvert.data[7] = (double)wiibb.BalanceBoardState.SensorValuesRaw.TopLeft;
		m_WiiConvert.data[8] = (double)wiibb.BalanceBoardState.SensorValuesRaw.TopRight;
		m_WiiConvert.data[9] = (double)wiibb.BalanceBoardState.SensorValuesRaw.BottomRight;
		m_WiiConvert.data[10]= (double)wiibb.BalanceBoardState.SensorValuesRaw.BottomLeft;
		m_WiiConvertOut.write();

		if(m_debug == true)
		{
			system("cls");
			std::cout << "A  : " << wiibb.mWiiRemoteState.ButtonState.A << std::endl;
			std::cout << "GX : " << wiibb.BalanceBoardState.CenterOfGravity.X << std::endl;
			std::cout << "GY : " << wiibb.BalanceBoardState.CenterOfGravity.Y << std::endl;
			std::cout << "Sensor Value Kg(Top left)     : " << wiibb.BalanceBoardState.SensorValuesKg.TopLeft << std::endl;
			std::cout << "Sensor Value Kg(Top right)    : " << wiibb.BalanceBoardState.SensorValuesKg.TopRight << std::endl;
			std::cout << "Sensor Value Kg(Bottom right) : " << wiibb.BalanceBoardState.SensorValuesKg.BottomRight << std::endl;
			std::cout << "Sensor Value Kg(Bottom left)  : " << wiibb.BalanceBoardState.SensorValuesKg.BottomLeft << std::endl;
			std::cout << "Sensor Value Raw(Top left)    : " << wiibb.BalanceBoardState.SensorValuesRaw.TopLeft << std::endl;
			std::cout << "Sensor Value Raw(Top right)   : " << wiibb.BalanceBoardState.SensorValuesRaw.TopRight << std::endl;
			std::cout << "Sensor Value Raw(Bottom right): " << wiibb.BalanceBoardState.SensorValuesRaw.BottomRight << std::endl;
			std::cout << "Sensor Value Raw(Bottom left) : " << wiibb.BalanceBoardState.SensorValuesRaw.BottomLeft << std::endl;
		}
	}
	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiBalanceBoardTranslator::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiBalanceBoardTranslator::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiBalanceBoardTranslator::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiBalanceBoardTranslator::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiBalanceBoardTranslator::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void WiiBalanceBoardTranslatorInit(RTC::Manager* manager)
  {
    coil::Properties profile(wiibalanceboardtranslator_spec);
    manager->registerFactory(profile,
                             RTC::Create<WiiBalanceBoardTranslator>,
                             RTC::Delete<WiiBalanceBoardTranslator>);
  }
  
};


