// -*- C++ -*-
/*!
 * @file  WiiBalanceBoardTranslator.h
 * @brief WiiBalanceBoardTranslator
 * @date  $Date$
 *
 * @author 〒135-8548
 * 東京都江東区豊洲3-7-5　芝浦工業大学　研究棟　11-Q-32
 * 水川(HRI)研究室
 * 代表者:水川　真   (Emai:mizukawa<at>sic.shibaura-it.ac.jp)
 * 作成者;田中　基雅(Email:m108081<at>shibaura-it.ac.jp)
 *          藤田　恒彦(Email:m109077<at>shibaura-it.ac.jp)
 *          鷹栖　尭大（Email:m109056<at>shibaura-it.ac.jp）
 *
 * WiiBalanceBoardTranslatorについての著作権は，芝浦工業大学 水川研究室に帰属しま
 * す．
 * 非営利目的利用は自由です．
 *
 * $Id$
 */

#ifndef WIIBALANCEBOARDTRANSLATOR_H
#define WIIBALANCEBOARDTRANSLATOR_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "WiiInfoStub.h"
#include "WiiBalanceBoardConvert.h"

// </rtc-template>

using namespace RTC;

/*!
 * @class WiiBalanceBoardTranslator
 * @brief WiiBalanceBoardTranslator
 *
 */
class WiiBalanceBoardTranslator
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  WiiBalanceBoardTranslator(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~WiiBalanceBoardTranslator();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * Configurationの初期化を行う
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * 
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * キャリブレーションデータを取得し，WiiRemoteAgentに送信する．
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre WiiリモコンにBalanceBoardを接続している事
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * WiiRemoteAgentから生データが入力された時に，それを整形データに変換し，出力す
   * る．
   * 整形データはTimedDoubleSeq型で出力する．
   * データの内訳を以下に記す．
   * [0] 電源ボタンの状態(BOOL 値)
   * [1] X軸の重心座標
   * [2] Y軸の重心座標
   * [3] 左上にかかる重量データ
   * [4] 右上にかかる重量データ
   * [5] 右下にかかる重量データ
   * [5] 左下にかかる重量データ
   * [7] 左上センサの生データ
   * [8] 右上センサの生データ
   * [9] 右下センサの生データ
   * [10] 左下センサの生データ
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * コンソール画面に整形データを表示するかどうかを選択するためのConfiguration
   * - Name:  debug
   * - DefaultValue: 0
   */
  bool m_debug;
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedOctetSeq m_WiiData;
  /*!
   * WiiRemoteAgentから，WiiリモコンとBalanceBoardの情報を受け取る．
   * - Type: TimedOctetSeq
   * - Number: 22Byte
   */
  InPort<TimedOctetSeq> m_WiiDataIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedDoubleSeq m_WiiConvert;
  /*!
   * Wiiリモコン本体，BalanceBoardの情報を整形したデータを出力する．
   * - Type: TimedDoubleSeq
   * - Number: 11
   */
  OutPort<TimedDoubleSeq> m_WiiConvertOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   * WiiRemoteAgentにWiiリモコン本体及びBalanceBoardのキャリブレーションデータを
   * 要求する．
   */
  RTC::CorbaPort m_WiiInfoPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  /*!
   */
  RTC::CorbaConsumer<WiiInfo> m_WiiInfo;
  
  // </rtc-template>

 private:
	 WiiBalanceBoardConvert wiibb;
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

};


extern "C"
{
  DLL_EXPORT void WiiBalanceBoardTranslatorInit(RTC::Manager* manager);
};

#endif // WIIBALANCEBOARDTRANSLATOR_H
