#include "WiiMotionPlus.h"

WiiMotionPlus::WiiMotionPlus(void)
{
	isExtension = false;
	LEDandVibrationCommand = 0x00;
}

WiiMotionPlus::~WiiMotionPlus(void)
{
}

bool WiiMotionPlus::Initialize(){

	if(!FindWiiRemote())
	{
		std::cerr << "not found wii remote" <<std::endl;
		return false;
	}

	ReadWiimoteCalibration();
	GetStatus();

	Report_0x12(0x00,0x31);
	InitializeExtension();
	isStartThread = true;
	hThread = (HANDLE)_beginthreadex(NULL, 0, this->OnReceive, this, 0, NULL);
	Sleep(500);

	return true;
}


void WiiMotionPlus::InitializeExtension(){

	BYTE *buff = ReadData(0x04a400fa, 6);
	if(!((wiiExtState[9]==buff[9])&(wiiExtState[10]==buff[10])&(wiiExtState[11]==buff[11])))
	{
		//gRg[̏R}h
		WriteData(0x04a400f0,0x55);
		Sleep(50);
		//gRg[̏R}h
		WriteData(0x04a400fb,0x00);
		Sleep(50);
		//Wii[VvX̏R}h
		WriteData(0x04a600f0,0x55);
		Sleep(50);
		//Wii[VvX̏R}h
		WriteData(0x04a600fe,0x04);
		Sleep(50);
	}

	buff = ReadData(0x04a400fa, 6);
	memcpy(wiiExtState,buff,22);
	long type = ((long)buff[9]) << 16 | ((long)buff[10]) << 8 | buff[11];

	ZeroMemory(&wiiRemoteExtensionCalibrationData, sizeof(wiiRemoteExtensionCalibrationData));

	if(type == 0x200405)
	{
		buff = ReadData(0x04a40020, 16);

		memcpy(wiiRemoteExtensionCalibrationData,buff,22);	//It's not known exactly how

		//gRg[{x{{^f[^̗vR}h
		Report_0x12(0x04,0x35);
	}
	else
	{
		std::cout << "[VvXڑĂ" << std::endl;
	}
	Sleep(50);
	return;
}

bool WiiMotionPlus::WiiRemoteThread(){
	Sleep(1);
	BYTE* buff = ReadReport();

	switch(buff[0])
	{
	case 0x20: // Xe[^X
		{
			// g
			isExtension = (buff[3] & 0x02) != 0;
			memcpy(wiiStateData,buff,22);
			if(isExtension)
			{
				InitializeExtension();
			}
			else
			{
				// gȂCvR}h𑗂
				ReadWiimoteCalibration();
				Report_0x12(0x00,0x31);
			}
			break;
		}
	case 0x31:
		{
			break;
		}
	default:
		//Report ID 35
		memcpy(mReadBuff,buff,22);	
		break;
	}
	return true;
}

bool WiiMotionPlus::Finalize(){
	CloseThread();
	return WiiClosed();
}

void WiiMotionPlus::CloseThread(){
	if(isStartThread == true)
	{
		if (hThread != NULL) 
		{
			isStartThread = false;
			//Xbh̏I҂
			WaitForSingleObject(hThread, INFINITE);

			hThread = NULL;
			CloseHandle(hThread);
		}
	}
}

void WiiMotionPlus::ReadWiimoteCalibration(){
	ZeroMemory(&wiiRemoteCalibrationData, sizeof(wiiRemoteCalibrationData));
	BYTE *buff = ReadData(0x0016, 7);
	memcpy(wiiRemoteCalibrationData,buff,22);
}

BYTE *WiiMotionPlus::ReadData(int address, int size){
	BYTE outputReport[22];
	BYTE readBuff[256];
	int counter = 0;
	int dataCount = 0;
	bool isResetEvent = false;

	// CXbhU~
	if(hThread)
	{
		isResetEvent = true;
		ResetEvent(hThread);
	}
	
	ZeroMemory(&outputReport, sizeof(outputReport));
	ZeroMemory(&readBuff, sizeof(readBuff));
	ZeroMemory(&mReadBuff_17, sizeof(mReadBuff_17));

	outputReport[0] = 0x17;
	outputReport[1] = ((address & 0xff000000) >> 24);
	outputReport[2] = ((address & 0x00ff0000) >> 16);
	outputReport[3] = ((address & 0x0000ff00) >>  8);
	outputReport[4] = (address & 0x000000ff);
	outputReport[5] = (size & 0xff00) >> 8;
	outputReport[6] = size & 0xff;
	WriteReport(outputReport);

	while(1)
	{
		ReadReport(readBuff,&inputLength);
 		if(counter > 100)
		{
			std::cout << "f[^̎MɎs܂D" << std::endl;
			break;
		}

		int buffsize = (readBuff[3] >> 4) + 1;
		int offset = (readBuff[4] << 8 | readBuff[5]);

		if(readBuff[0] == 0x21)
		{
			if(dataCount * 22 > 255)
			{
				std::cout << "obt@TCY܂" << std::endl;
				break;
			}
			memcpy(mReadBuff_17 + dataCount * 22,readBuff,inputLength);
			dataCount++;
			if(((address & 0xffff) + size) == (buffsize + offset))
			{
				break;
			}
		}
		counter++;
	}
	
	if(isResetEvent)
	{
		SetEvent(hThread);
	}

	return mReadBuff_17;
}

unsigned __stdcall WiiMotionPlus::OnReceive(void* myself){
	WiiMotionPlus &wiiMo = *(WiiMotionPlus*)myself;
	while(1){
		wiiMo.WiiRemoteThread();
		if(wiiMo.isStartThread == false ){
			break;
		}
	}
	return 0;
}

bool WiiMotionPlus::SetLED_Vibration(BYTE setCommand){
	bool isResetEvent = false;
	LEDandVibrationCommand = setCommand;

	// CXbhU~
	if(hThread)
	{
		isResetEvent = true;
		ResetEvent(hThread);
	}

	BYTE command[2];
	command[0] = 0x11;
	command[1] = setCommand;

	WriteReport(command);

	if(isResetEvent)
	{
		SetEvent(hThread);
	}

	return true;
}

bool WiiMotionPlus::RequestCalib(BYTE* input){
	memcpy(input,wiiRemoteCalibrationData,22);
	return true;
}

bool WiiMotionPlus::RequestCalibExt(BYTE *input){
	memcpy(input,wiiRemoteExtensionCalibrationData,22);
	return true;
}

bool WiiMotionPlus::RequestData(BYTE *input){
	memcpy(input,mReadBuff,22);
	//DebugWriteLine(input,22);
	return true;
}

bool WiiMotionPlus::RequestState(BYTE *input){
	GetStatus();
	Sleep(15);
	//obt@܂Ă邱Ƃ\z邽߁C_ōXV
	memcpy(input, wiiStateData,22);
	return true;
}