// -*- C++ -*-
/*!
 * @file  WiiRemoteAgent.cpp
 * @brief WiiRemoteAgent
 * @date $Date$
 *
 * @author 〒135-8548
 * 東京都江東区豊洲3-7-5　芝浦工業大学　研究棟　11-Q-32
 * 水川(HRI)研究室
 * 代表者:水川　真   (Emai:mizukawa<at>sic.shibaura-it.ac.jp)
 * 作成者;田中　基雅(Email:m108081<at>shibaura-it.ac.jp)
 *           藤田　恒彦(Email:m109077<at>shibaura-it.ac.jp)
 *           鷹栖　尭大（Email:m109056<at>shibaura-it.ac.jp）
 *
 * Microsoftより提供されているWindows Driver Kitのライブラリを除き，芝浦工業大学
 * 水川研究室に帰属します．
 * 非営利目的利用は自由です．
 *
 * $Id$
 */

#include "WiiRemoteAgent.h"

// Module specification
// <rtc-template block="module_spec">
static const char* wiiremoteagent_spec[] =
  {
    "implementation_id", "WiiRemoteAgent",
    "type_name",         "WiiRemoteAgent",
    "description",       "WiiRemoteAgent",
    "version",           "1.0.0",
    "vendor",            "Shibaura Institute of Technology",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    //"exec_cxt.periodic.rate", "1.0",
    // Configuration variables
    "conf.default.ClassicController", "0",
    "conf.default.Nunchaku", "0",
    "conf.default.BalanceBoard", "0",
    "conf.default.MotionPlus", "0",
    "conf.default.OtherExt", "0",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
WiiRemoteAgent::WiiRemoteAgent(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_commandIn("Command", m_command),
    m_wiiOut("Wii", m_wii),
    m_WiiInfoPort("WiiInfo")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
WiiRemoteAgent::~WiiRemoteAgent()
{
}


/*!
 * Wiiリモコンの検出
 */
RTC::ReturnCode_t WiiRemoteAgent::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("Command", m_commandIn);
  
  // Set OutPort buffer
  registerOutPort("Wii", m_wiiOut);
  
  // Set service provider to Ports
  m_WiiInfoPort.registerProvider("WiiInfo", "WiiInfo", m_WiiInfo);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_WiiInfoPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("ClassicController", m_ClassicController, "0");
  bindParameter("Nunchaku", m_Nunchaku, "0");
  bindParameter("BalanceBoard", m_BalanceBoard, "0");
  bindParameter("MotionPlus", m_MotionPlus, "0");
  bindParameter("OtherExt", m_OtherExt, "0");
  
  m_WiiInfo.m_valueWiiRemoteCalibration.length(22);
  m_WiiInfo.m_valueWiiExtensionCalibration.length(22); // 拡張コントローラ用キャリブレーション
  m_WiiInfo.m_valueWiiBalanceCalibration.length(44); // バランスボードのキャリブレーション

  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiRemoteAgent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiRemoteAgent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t WiiRemoteAgent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * Wiiリモコンのハンドルを開く．
 * また，Configurationの設定がされていた場合，拡張コントローラの初期化コマンドの送信
 * と共にキャリブレーションデータの取得を行う．
 */

RTC::ReturnCode_t WiiRemoteAgent::onActivated(RTC::UniqueId ec_id)
{
	bool isExtController = true;
	bool isBalanceBoard = false;

	//コンフィギュレーションのアップデート
	m_configsets.update();

	//デザインパターンのストラテジーを適用
	//新たな機能を追加したい場合，クラスの追加とコンフィギュレーションの追加だけで利用可能になる

	//コンフィギュレーションのバランスボードがtrueかどうか
	if(m_BalanceBoard == true)
	{
		isBalanceBoard = true;
		wiimote = new WiiBalanceBoard();
	}
	//コンフィギュレーションのヌンチャクがtrueかどうか
	else if(m_Nunchaku == true)
	{
		wiimote = new WiiNunchaku();
	}
	//コンフィギュレーションのモーションプラスがtrueかどうか
	else if(m_MotionPlus ==true)
	{
		wiimote = new WiiMotionPlus();
	}
	//コンフィギュレーションのクラシックコントローラがtrueかどうか
	else if(m_ClassicController == true)
	{
		wiimote = new WiiClassicController();
	}
	else if(m_OtherExt == true)
	{
		//現在未対応．エラー状態に遷移させる
		std::cout << "Unsupported Option" << std::endl;
		return RTC::RTC_ERROR;
	}
	//上記，以外の設定の場合
	else
	{
		isExtController = false;
		wiimote = new WiiBase();
	}

	//wiiリモコンの初期化処理
	if(!wiimote->Initialize()){
		//Wiiリモコンの接続失敗(エラーに遷移)
		return RTC::RTC_ERROR;
	}
	//コンフィギュレーションの値を読み込んでif文で処理の変更が必要
	BYTE buff[255];

	//バランスボード以外の設定の場合
	if(!isBalanceBoard)
	{
		while(1){
			//キャリブレーションデータの要求コマンド
			wiimote->RequestCalib(buff);
			//キャリブレーションデータが取得できるまでループ
			if(buff[0]==0x21){
				break;
			}	
		}

		//キャリブレーションデータをサービスポートに格納
		for(int i = 0; i < 22; i++){
			m_WiiInfo.m_valueWiiRemoteCalibration[i] = buff[i];
		}
	}

	//拡張コントローラが接続されていた場合
	if(isExtController)
	{
		while(1){
			//キャリブレーションデータの要求コマンド
			wiimote->RequestCalibExt(buff);
			//キャリブレーションデータが取得できるまでループ
			if(buff[0]==0x21){
				break;
			}	
		}

		//バランスボードの場合，キャリブレーションデータが44BYTEになるため変更
		if(isBalanceBoard)
		{
			for(int i = 0; i < 44; i++){
				m_WiiInfo.m_valueWiiBalanceCalibration[i]= buff[i];
			}
		}
		else
		{
			//キャリブレーションデータをサービスポートに格納
			for(int i = 0; i < 22; i++){
				m_WiiInfo.m_valueWiiExtensionCalibration[i]= buff[i];
			}
		}
	}

	return RTC::RTC_OK;
}

/*!
 * Wiiリモコンのハンドルを閉じる
 */

RTC::ReturnCode_t WiiRemoteAgent::onDeactivated(RTC::UniqueId ec_id)
{
	//Wiiリモコンの終了処理
	wiimote->Finalize();
	//wiimoteのデリート
	delete wiimote;

	//キャリブレーションデータを初期化
	for(int i = 0; i < 44; i++){
		m_WiiInfo.m_valueWiiBalanceCalibration[i]= 0;
	}

	for(int i = 0; i < 22; i++){
		m_WiiInfo.m_valueWiiExtensionCalibration[i]= 0;
	}
	return RTC::RTC_OK;
}

/*!
 * Wiiリモコンのデータ取得を行い，アウトポートにより出力を行う．
 * インポートからのデータを受け，LED及びバイブレーション機能の制御を行う．
 */

RTC::ReturnCode_t WiiRemoteAgent::onExecute(RTC::UniqueId ec_id)
{
	//インポートの受信確認
	if( m_commandIn.isNew()){
		//データの確認
		m_commandIn.read();
		//LEDと振動の制御を行う
		wiimote->SetLED_Vibration(m_command.data);
	}

	BYTE buff[255];
	//Wiiリモコンのデータを取得する
	wiimote->RequestData(buff);

	m_wii.data.length(22);
	//取得データの格納
	for(int i = 0; i < 22; i++){
		m_wii.data[i] = buff[i];
	}

	//アウトポートへ書き込み
	m_wiiOut.write();

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiRemoteAgent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * エラー状態の表示を行う．
 */

RTC::ReturnCode_t WiiRemoteAgent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiRemoteAgent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t WiiRemoteAgent::onStateUpdate(RTC::UniqueId ec_id)
{
	BYTE buff[255];
	//Wiiリモコンのステータス取得
	wiimote->RequestState(buff);
	//バッテリ情報をサービスポートに格納
	m_WiiInfo.m_valueBattery = buff[6] / 2;

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t WiiRemoteAgent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void WiiRemoteAgentInit(RTC::Manager* manager)
  {
    coil::Properties profile(wiiremoteagent_spec);
    manager->registerFactory(profile,
                             RTC::Create<WiiRemoteAgent>,
                             RTC::Delete<WiiRemoteAgent>);
  }
  
};


