// -*- C++ -*-
/*!
 * @file  WiiRemoteAgent.h
 * @brief WiiRemoteAgent
 * @date  $Date$
 *
 * @author 〒135-8548
 * 東京都江東区豊洲3-7-5　芝浦工業大学　研究棟　11-Q-32
 * 水川(HRI)研究室
 * 代表者:水川　真   (Emai:mizukawa<at>sic.shibaura-it.ac.jp)
 * 作成者;田中　基雅(Email:m108081<at>shibaura-it.ac.jp)
 *           藤田　恒彦(Email:m109077<at>shibaura-it.ac.jp)
 *           鷹栖　尭大（Email:m109056<at>shibaura-it.ac.jp）
 *
 * Microsoftより提供されているWindows Driver Kitのライブラリを除き，芝浦工業大学
 * 水川研究室に帰属します．
 * 非営利目的利用は自由です．
 *
 * $Id$
 */

#ifndef WIIREMOTEAGENT_H
#define WIIREMOTEAGENT_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "WiiInfoSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// Wii Remote Control Class header
#include "WiiBase.h"
#include "WiiNunchaku.h"
#include "WiiBalanceBoard.h"
#include "WiiMotionPlus.h"
#include "WiiClassicController.h"

// </rtc-template>

using namespace RTC;

/*!
 * @class WiiRemoteAgent
 * @brief WiiRemoteAgent
 *
 */
class WiiRemoteAgent
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  WiiRemoteAgent(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~WiiRemoteAgent();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   * Wiiリモコンの検出
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre 東芝製のBluetoothStackドライバにWiiリモコンを登録する
   * @post 最初に登録されたWiiリモコンまたはバランスボードが接続されるため，新規
   * に別のWiiリモコンを登録する場合は以前利用していたWiiリモコンやバランスボードを登
   * 録から削除する事
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   * 
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   * Wiiリモコンのハンドルを開く．
   * また，Configurationの設定がされていた場合，拡張コントローラの初期化コマンドの送信
   * と共にキャリブレーションデータの取得を行う．
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * @pre Configurationの設定により使用するコントローラを指定する．
   * ※Activate中のコントローラ変更は未実装
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   * Wiiリモコンのハンドルを閉じる
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   * Wiiリモコンのデータ取得を行い，アウトポートにより出力を行う．
   * インポートからのデータを受け，LED及びバイブレーション機能の制御を行う．
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   * エラー状態の表示を行う．
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * Wiiリモコンの拡張コントローラであるClassicControllerを利用するかどうか決定す
   * るConfiguration
   * - Name:  ClassicController
   * - DefaultValue: 0
   * - Constraint: 複数の拡張コントローラには未対応
   */
  bool m_ClassicController;
  /*!
   * Wiiリモコンの拡張コントローラであるNunchakuを利用するかどうか決定するConfig
   * uration
   * - Name:  Nunchaku
   * - DefaultValue: 0
   * - Constraint: 複数の拡張コントローラには未対応
   */
  bool m_Nunchaku;
  /*!
   * Wiiリモコンの拡張コントローラであるBalanceBoardを利用するかどうか決定するCo
   * nfiguration
   * - Name:  BalanceBoard
   * - DefaultValue: 0
   * - Constraint: 複数の拡張コントローラには未対応
   */
  bool m_BalanceBoard;
  /*!
   * Wiiリモコンの拡張コントローラであるMotionPlusを利用するかどうか決定するConf
   * iguration
   * - Name:  MotionPlus
   * - DefaultValue: 0
   * - Constraint: 複数の拡張コントローラには未対応
   */
  bool m_MotionPlus;
  /*!
   * その他の拡張コントローラを接続した場合に利用するコンフィギュレーション(現状
   * ：未実装のため利用できない)
   * - Name:  OtherExt
   * - DefaultValue: 0
   * - Constraint: 複数の拡張コントローラには未対応
   */
  bool m_OtherExt;
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedOctet m_command;
  /*!
   * WiiリモコンのLEDとバイブレーション機能を制御するポート
   * 1Byte：(上位4bit：LEDの点灯パターン，最下位ビット：振動のON/OFF)
   * - Type: TimedOctet
   * - Number: 2Byte
   */
  InPort<TimedOctet> m_commandIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedOctetSeq m_wii;
  /*!
   * Wiiリモコンのデータ送信用ポート
   * 生データの22Byteを送信する
   * - Type: TimedOctetSeq
   * - Number: 22Byte
   */
  OutPort<TimedOctetSeq> m_wiiOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   * Wiiリモコンの情報を提供するサービスポート
   * Interface: Interface: キャリブレーションデータやバッテリ情報の提供を行う
   */
  RTC::CorbaPort m_WiiInfoPort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  /*!
   * Wiiリモコンの情報を取得するProvider用のポート
   */
  WiiInfoSVC_impl m_WiiInfo;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  
  // </rtc-template>

 private:
	 CWiiRemoteCntl *wiimote;
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

};


extern "C"
{
  DLL_EXPORT void WiiRemoteAgentInit(RTC::Manager* manager);
};

#endif // WIIREMOTEAGENT_H
